-- Version 2.0, August 3rd 2020
-- Version 2.01, January 19th 2021 ->Made the loop lighter. Will only loop at 0.05 seconds if at least one sound is waiting in the stack.


-- Global variable
FSQ = {}
FSQ.SoundToAllTable = {}
FSQ.RadioMessageTable = {}
FSQ.Debug = false

endSoundTime = 0;
endRadioSoundTime = 0;

-- Example FSQ.AddRadioMessageToAll("message.ogg",2.5,"myUnit","FM",255) for an FM message
-- Example FSQ.AddRadioMessageToAll("message.ogg",2.5,"myUnit","AM",127) for a AM message
function FSQ.AddRadioCall(aSoundfile,aDuration,aUnitName,aModulation,aFrequency)

  local _radioBroadcast = {soundFile = aSoundfile, duration = aDuration, message="",messageShownTime=0, unitName = aUnitName, modulation = aModulation, frequency = aFrequency}
  table.insert(FSQ.RadioMessageTable,_radioBroadcast)
  if (FSQ.Debug) then
    trigger.action.outText("FSQ: Adding radio broadcast "..aSoundfile, 5)
  end

end

function FSQ.AddRadioCallM(aSoundfile,aDuration,aUnitName,aModulation,aFrequency, aUIMessage,aUIShowTime)

  local _radioBroadcast = {soundFile = aSoundfile, duration = aDuration, message=aUIMessage,messageShownTime=aUIShowTime, unitName = aUnitName, modulation = aModulation, frequency = aFrequency}
  table.insert(FSQ.RadioMessageTable,_radioBroadcast)
  if (FSQ.Debug) then
    trigger.action.outText("FSQ: Adding radio broadcast "..aSoundfile.." with message "..aUIMessage, 5)
  end

end


function FSQ.AddSoundToAll(aSoundfile,aDuration)

  local _sound = {soundFile = aSoundfile, duration = aDuration, message="",messageShownTime=0}
  table.insert(FSQ.SoundToAllTable,_sound)
  if (FSQ.Debug) then
    trigger.action.outText("FSQ: Adding sound "..aSoundfile, 5)
  end

end


function FSQ.AddSoundToAllM(aSoundfile,aDuration,aUIMessage,aUIShownTime)

  local _sound = {soundFile = aSoundfile, duration = aDuration, message = aUIMessage, messageShownTime = aUIShownTime}
  table.insert(FSQ.SoundToAllTable,_sound)
  if (FSQ.Debug) then
    trigger.action.outText("FSQ: Adding sound "..aSoundfile.." with message "..aUIMessage, 5)
  end

end



function FSQ.SoundEngine()

	local _count = table.getn(FSQ.SoundToAllTable)
	if (FSQ.Debug) then
		trigger.action.outText("FSQ: Sound in stack "..tostring(_count), 5)
	end
	
	if (_count>0) then

		local _currentTime = timer.getTime()
		if (FSQ.Debug) then
			trigger.action.outText("FSQ: Current time "..tostring(_currentTime), 5)
		end

		
		--Check if sound is currently playing
		if ((endSoundTime~=0) and (endSoundTime<_currentTime)) then
			endSoundTime=0

		end
		
		--Play a new sound if none are playing
		if (endSoundTime==0) and (_count>0) then
			local _soundToPlay = FSQ.SoundToAllTable[1]
			local _soundLength = _soundToPlay.duration
			local _soundFile = USERSOUND:New(_soundToPlay.soundFile)
			local _message = _soundToPlay.message
			local _messageTime = _soundToPlay.messageShownTime
			
			endSoundTime=timer.getTime()+_soundLength
			_soundFile:ToAll()
			table.remove(FSQ.SoundToAllTable, 1)


			if (FSQ.Debug) then
				trigger.action.outText("FSQ: Playing sound ".._soundToPlay.soundFile, 1)
			end
			
			if (_message~="") then
				MessageToAll(_message,_messageTime,"")
			
				
				if (FSQ.Debug) then
					trigger.action.outText("FSQ: Playing sound with message ".._message, 1)
				end
				
			end
			

		end
		
	end
	
	-- Schedule the next call
	if (_count>0) then
		timer.scheduleFunction(FSQ.SoundEngine, nil, timer.getTime() + 0.05) --run 5 times a seconds when the stack is filled
	else
		timer.scheduleFunction(FSQ.SoundEngine, nil, timer.getTime() + 1) --check once a second when the stack is empty
	end
  
 end
 
 function FSQ.RadioEngine()

	local _count = table.getn(FSQ.RadioMessageTable)
	if (FSQ.Debug) then
		trigger.action.outText("FSQ: Radio in stack "..tostring(_count), 5)
	end
	
	if (_count>0) then

		local _currentRadioTime = timer.getTime()
		if (FSQ.Debug) then
			trigger.action.outText("FSQ: Current time "..tostring(_currentRadioTime), 5)
		end

		
		--Check if sound is currently playing
		if ((endRadioSoundTime~=0) and (endRadioSoundTime<_currentRadioTime)) then
			endRadioSoundTime=0
		end
		
		--Play a new radio message if none are playing
		if (endRadioSoundTime==0) and (_count>0) then
			local _radioCallToPlay = FSQ.RadioMessageTable[1]
			local _unit = UNIT:FindByName(_radioCallToPlay.unitName)
			if (_unit~=nil) then

				local _soundLength = _radioCallToPlay.duration
				local _soundFile = _radioCallToPlay.soundFile
				local _message = _radioCallToPlay.message
				local _messageTime = _radioCallToPlay.messageShownTime
				local _unitRadio = _unit:GetRadio()
				--local _unitRadio = unitTest:GetRadio()
				local _modulation = _radioCallToPlay.modulation
				local _frequency = _radioCallToPlay.frequency
				
				endRadioSoundTime=timer.getTime()+_soundLength
				if (_modulation=="AM") then
					_unitRadio:SetFileName(_soundFile):SetFrequency(_frequency):SetModulation(radio.modulation.AM):SetLoop(false)
				end

				if (_modulation=="FM") then
					_unitRadio:SetFileName(_soundFile):SetFrequency(_frequency):SetModulation(radio.modulation.FM):SetLoop(false)
				end
				--Broadcast
				_unitRadio:Broadcast()
				table.remove(FSQ.RadioMessageTable, 1)

				if (FSQ.Debug) then
					trigger.action.outText("FSQ: Playing radio call ".._soundFile, 1)
				end
				
				if (_message~="") then
					MessageToAll(_message,_messageTime,"")
				
					
					if (FSQ.Debug) then
						trigger.action.outText("FSQ: Playing radio call ".._soundFile.." with message ".._message, 1)
					end
					
				end
			end

		end
		
	end
	
	-- Schedule the next call
	if (_count>0) then
		timer.scheduleFunction(FSQ.RadioEngine, nil, timer.getTime() + 0.05) --run 5 times a seconds when the stack is filled
	else
		timer.scheduleFunction(FSQ.RadioEngine, nil, timer.getTime() + 1) --check once a second when the stack is empty
	end
  
 end
 
 -- Start the sound engine
 timer.scheduleFunction(FSQ.SoundEngine, nil, timer.getTime() + 2)
 timer.scheduleFunction(FSQ.RadioEngine, nil, timer.getTime() + 2)